local ADDON_NAME, Core = ...

--<
local UI = Core.UI
local SM = Core.SM
local myInfo = Core.myInfo
local utils = Core.utils
-->

local TS = "%I:%M "
local TS_NONE = ""
local TS_GREY = "|cFFB3B3B3%I:%M |r"
local FORMAT_GENERIC = "%s[%s]: %s"


local log = setmetatable({}, {
	__newindex = function(self, guid, msg)
		log[guid] = {msg}
		--MessageHandler:NewMessage(guid, msg)
	end
})

local registry = {
	privateEvents = {
		["CHAT_MSG_DND"] = true,
		["CHAT_MSG_AFK"] = true,
		["CHAT_MSG_IGNORED"] = true,
		["CHAT_MSG_WHISPER_INFORM"] = true,
		["CHAT_MSG_WHISPER"] = true,
		["CHAT_MSG_BN_WHISPER"] = true,
		["CHAT_MSG_BN_WHISPER_INFORM"] = true,
	},
}

local handler = CreateFrame("Frame")
handler:RegisterEvent("CHAT_MSG_FILTERED")
handler:RegisterEvent("CHAT_MSG_RESTRICTED")
for ev, _ in pairs(registry.privateEvents) do
	handler:RegisterEvent(ev)
end

local prefixes = {
	EMOTE = "%s* %s ",
	
	classic = {
		WHISPER = "%s".. CHAT_WHISPER_GET,
		WHISPER_INFORM = "%s".. CHAT_WHISPER_INFORM_GET,
		BN_WHISPER = "%s".. CHAT_WHISPER_GET,
		BN_WHISPER_INFORM = "%s".. CHAT_WHISPER_INFORM_GET,
		SAY = "%s".. CHAT_SAY_GET
	}
}


local function CheckForPatternHighlight (ctype, msg)
	local pattern, hex
	
	if ctype ~= "SAY" then
		pattern = '%b""'; hex = RGBToColorCode(utils.GetRGB("SAY"))
	elseif ctype ~= "EMOTE" then
		pattern = '%b**'; hex = RGBToColorCode(utils.GetRGB("EMOTE"))
	end
	
	msg = msg:gsub(pattern, function(catch)
		local _, pos = msg:find(catch)
		
		if pos and msg:sub(pos+1, pos+2) == "|r" then
			return catch
		else
			return format("%s%s|r", hex, catch)
		end
	end)
	
	return msg
end


local function Filter (event, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15, A16)
	local chatFilters = ChatFrame_GetMessageEventFilters(event)
	local filtered = false
	
	if chatFilters then
		local a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16
		
		for _, func in next, chatFilters do
			filtered, a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16 = func(handler, event, A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13,A14,A15,A16)
			
			if filtered then
				return true
				
			elseif a2 then
				A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13,A14,A15,A16 = a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16
			end
		end
	end
	
	return filtered, A1,A2,A3,A4,A5,A6,A7,A8,A9,A10,A11,A12,A13,A14,A15,A16
end


local function ColouriseName (e, a1, author, a2,a3,a4,a5,a6,a7,a8,a9,a10, guid)
	local _, engClass = GetPlayerInfoByGUID(guid)
	local colour = CreateColor(RAID_CLASS_COLORS[engClass])
	return colour:WrapTextInColor(author)
end

handler["WHISPER"] = function (event, author, guid)
	local displayName = GetCharacterName(event, author, guid)
	return true, GetPlayerLink(author, displayName), SM.CreateFromGUID(guid)
end


handler["WHISPER_INFORM"] = function (event, author, guid)
	local displayName = Core.db.UseVerbose and GetCharacterName(event, author, guid) or GetCharacterName(event, myInfo.character, myInfo.GUID)
	local playerLink = GetPlayerLink(Core.db.UseVerbose and author or myInfo.character, displayName)
	return false, playerLink, SM.CreateFromGUID(guid)
end


handler["BN_WHISPER"] = function (event, author, guid, bnetID)
	return true, GetBNPlayerLink(author, author, bnetID), SM.CreateFromID(bnetID)
end


handler["BN_WHISPER_INFORM"] = function (event, author, guid, bnetID)
	local playerLink = Core.db.UseVerbose and GetBNPlayerLink(author, author, bnetID) or myInfo.battleTag
	return false, playerLink, SM.CreateFromID(bnetID)
end


function Core.RegisterForChatType (session, ctype)
	registry[ctype] = registry[ctype] or {}
	table.insert(registry[ctype], session)
end

local function RegisterForChatEvent (e)
	handler:RegisterEvent(e)
end

handler:SetScript("OnEvent", function(self, e, unfilteredMsg, ...)
	local filtered, msg, author, lang, chan, target, flag, zone, chanID, a9, a10, lineID, guid, bnetID = Filter(e, unfilteredMsg, ...)
	if filtered then
		return
	end


	local ctype = e:gsub("CHAT_MSG_", "")

	if ctype ~= "TEXT_EMOTE" and Core.db.ForceHighlight then
		msg = CheckForPatternHighlight(ctype, msg)
	end
	
	--msg = C_ChatInfo.ReplaceIconAndGroupExpressions(msg, false, false)

	if not registry.privateEvents[e] then

	else
		SM:UpdateIfNeeded(author, guid)
		
		local inbound = (not ctype:match("INFORM$") and true) or false
		local sessionTab, playerLink, displayName
		
		if ctype:match("BN_") then
			sessionTab = SM:Get(bnetID)
			if not sessionTab then
				sessionTab = SM.CreateFromID(bnetID)
			end
			playerLink = inbound and GetBNPlayerLink(author, author, bnetID, lineID, ctype) or myInfo.battleTag

		else
			sessionTab = SM:Get(guid)
			if not sessionTab then
				sessionTab = SM.CreateFromGUID(guid)
			end
			
			local nameForLink = inbound and author or myInfo.fullName
			local guidForLink = inbound and guid or myInfo.GUID
			
			if sessionTab.preferences.nameColouredByClass then
				displayName = GetColoredName(e, msg, nameForLink, lang,nil,target,flag,nil, chan, nil,nil,lineID, guidForLink)
				playerLink = inbound and GetPlayerLink(author, displayName) or displayName
				
			else
				playerLink = inbound and GetPlayerLink(author, Ambiguate(author, "none")) or myInfo.character
			end
		end
		

		local prefs = sessionTab.preferences
		local ts
		
		if prefs.tsEnabled then
			ts = prefs.tsCustomColour and date(TS_GREY) or date(TS)
		else
			ts = TS_NONE
		end

		local body = FORMAT_GENERIC:format(ts, playerLink, msg)
		local r, g, b
		
		if prefs.textCustomColour and (guid == myInfo.guid or not(inbound)) then
			r, g, b = Core.colours.greyText:GetRGB()
		else
			local info =  ChatTypeInfo[ctype]
			r, g, b = info.r, info.g, info.b
		end

		sessionTab.page:AddMessage(body, r, g, b)
		
		if UI.leftFrame:IsShown() then
			if not sessionTab:IsSelected() then
				sessionTab:UpdateNotice(ctype)
				sessionTab:StartGlowing()
			end
			
		elseif InCombatLockdown() or Core.db.KeepMinimized then
			if inbound then
				sessionTab:UpdateNotice(ctype)
				UI.leftFrame:Show()
				UI.main:Hide()
			end
		else
			SM:Switch(sessionTab)
			UI.leftFrame:Show()
		end
		
		if inbound or SM.lastWhisperTarget == nil then
			SM.lastWhisperTarget = sessionTab
			PlaySound(3081, "Master")
			FlashClientIcon()
		end
		
		if sessionTab.tab.tabIndex ~= 1 then
			table.insert(Core.orderedTabs, 1, table.remove(Core.orderedTabs, sessionTab.tab.tabIndex))
			Core.Tab_ReanchorAll()
		end
		
		if not bnetID and sessionTab.isUsingSM == nil then
			--sessionTab:Handshake()
		end
		
		
		
		
		local shitFucked = true
		if shitFucked then return end

		
		if inbound and bnetID then
			S = SM:Get(netID)
			playerLink = GetBNPlayerLink()
		end
	end
	
	
	
	local handlerFunction = self[ctype]
	
	if handlerFunction then
		-- DM session
		local inbound, playerLink, S = handlerFunction(e, author, guid, bnetID)
		local body = (prefixes[ctype] or PREFIX_GENERIC):format(ts, playerLink).. msg
		local r, g, b = utils.GetRGB(ctype)
		
		S.page:AddMessage(body, r, g, b, now)
		if Core.db.HistoryExpiry > 0 then
			--S:AddHistory(inbound, bnetID and true or false, ts, msg)
		end
		
		if S.isMuted then return end
		
		if UI:IsShown() then
			if not S:IsSelected() then
				S:UpdateNotice(ctype)
				S:StartGlowing()
			end
			
		elseif InCombatLockdown() or Core.db.KeepMinimized then
			if inbound then
				S:UpdateNotice(ctype)
			end
		else
			SM:Switch(S)
			UI:Show()
		end
		
		if inbound or SM.lastWhisperTarget == nil then
			SM.lastWhisperTarget = S
			PlaySound(3081, "Master")
			FlashClientIcon()
		end
		
		if S.tab.tabIndex ~= 1 then
			table.insert(Core.orderedTabs, 1, table.remove(Core.orderedTabs, S.tab.tabIndex))
			Core.Tab_ReanchorAll()
		end
		
		if not bnetID and S.isUsingSM == nil then
			S:Handshake()
		end
		
	else
		-- Public session(s)
		local playerLink = GetPlayerLink(author, GetCharacterName(e, author, guid))
		
		for i=1, #registry[ctype] do
			local S = registry[ctype][i]
			
			if (S.isDM and S.id == guid) or not S.isDM then
				S.page:AddMessage()
				
				if (not S:IsSelected() or not UI:IsShown()) and not S.isMuted then
					S:UpdateNotice(ctype)
				end
			end
			
		end
	end
end)
