
-- original main UI init
local function drawMainFrame ()
	local self = UI

	self:SetResizable(true)
	self:SetResizeBounds(450, 140, 1200, 800)
	self:SetFrameStrata("HIGH")
	self:SetScript("OnDragStart", self.StartMoving)
	self:SetScript("OnDragStop", main_OnDragStop)
	self:HookScript("OnSizeChanged", main_OnSizeChanged)
	self:SetScript("OnShow", main_OnShow)
	self:SetScript("OnHide", main_OnHide)
	self:SetBackdrop({
		edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
		
	})
	
	self.lowerFrame = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.lowerFrame:SetHeight(3*Core.LINE_HEIGHT + 8)
	self.lowerFrame:SetPoint("BOTTOMRIGHT")
	self.lowerFrame:SetPoint("BOTTOMLEFT", 110, 1)
	self.lowerFrame.BG = self.lowerFrame:CreateTexture(nil, "BACKGROUND")
	self.lowerFrame.BG:SetPoint("TOPLEFT", 2, -2)
	self.lowerFrame.BG:SetPoint("BOTTOMRIGHT", -2, 2)
	self.lowerFrame.BG:SetColorTexture(1, 1, 1)
	self.lowerFrame.BG:SetGradient("HORIZONTAL", CreateColor(0,0,0,0), CreateColor(0,0,0,0))
	self.lowerFrame.BG:Hide()
	self.lowerFrame:SetBackdrop({
		--edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
		--insets={left=2, right=2, top=2, bottom=2}
	})
	
	---------------
	-- EDITBOX
	---------------
	
	self.editBoxViewport = Core.LINE_HEIGHT*3
	
	self.editBoxScroller = CreateFrame("ScrollFrame", nil, self.lowerFrame)
	self.editBoxScroller:SetPoint("TOPLEFT", 6, -4)
	self.editBoxScroller:SetPoint("BOTTOMRIGHT", -4, 4)
	self.editBoxScroller:EnableMouseWheel(true)
	
	self.editBoxSlider = CreateFrame("Slider", nil, self.editBoxScroller)
	self.editBoxSlider:SetPoint("TOPRIGHT", 3, -2)
	self.editBoxSlider:SetPoint("BOTTOMRIGHT", 3, 2)
	self.editBoxSlider.knob = self:CreateTexture(nil)
	self.editBoxSlider.knob:SetSize(2, 12)
	self.editBoxSlider:SetThumbTexture(self.editBoxSlider.knob)
	self.editBoxSlider.knob:SetColorTexture(.8, .8, .8)
	self.editBoxSlider:SetOrientation("VERTICAL")
	self.editBoxSlider:SetSize(10, self.editBoxViewport)
	self.editBoxSlider:SetMinMaxValues(0, 0)
	self.editBoxSlider:SetValue(0)
	self.editBoxSlider:SetValueStep(Core.LINE_HEIGHT)
	self.editBoxSlider:SetScript("OnValueChanged", function(this, v) self.editBoxScroller:SetVerticalScroll(v) end)
	self.editBoxScroller:SetScript("OnMouseWheel", UI.OnMouseWheelEvent)

	self.editBox = CreateFrame("EditBox", ADDON_NAME.."MainEB", self.editBoxScroller)
	self.editBox:SetHeight(self.editBoxViewport)
	self.editBox:SetPoint("TOPLEFT")
	self.editBox:SetHitRectInsets(0, 0, 0, -self.editBoxViewport + Core.LINE_HEIGHT)
	self.editBox:SetAutoFocus(false)
	self.editBox:SetMultiLine(true)
	self.editBox:SetFontObject("ChatFontNormal")
	self.editBox:SetScript("OnEscapePressed", self.editBox.ClearFocus)
	self.editBox:SetScript("OnEditFocusGained", editBox_OnEditFocusGained)
	self.editBox:SetScript("OnEditFocusLost", editBox_OnEditFocusLost)
	self.editBox:SetScript("OnCursorChanged", function(this, _, y) self:OnCursorEvent(y) end)
	self.editBox:SetScript("OnEnter", editBox_CharacterCounter)
	self.editBox:SetScript("OnLeave", function(this, m) if m then self.editBox.counter:Hide() end end)
	self.editBox:SetScript("OnSizeChanged", editBox_OnSizeChanged)
	self.editBox:SetScript("OnTabPressed", editBox_OnTabPressed)
	self.editBox:SetScript("OnEnterPressed", editBox_OnEnterPressed)
	self.editBox:SetScript("OnSpacePressed", editBox_OnSpacePressed)
	self.editBox:SetScript("OnKeyDown", editBox_OnKeyDown)
	self.editBox.AddOrRemoveOnTextHandler = editBox_AddOrRemoveOnTextHandler

	self.editBox.counter = CreateFrame("Frame", nil, self.lowerFrame)
	self.editBox.counter:SetFrameStrata("DIALOG")
	self.editBox.counter:SetSize(35, 11)
	self.editBox.counter:SetPoint("TOPRIGHT", -5, 5)
	self.editBox.counter.BG = self.editBox.counter:CreateTexture(nil, "OVERLAY")
	self.editBox.counter.BG:SetColorTexture(.7, .25, 0)
	self.editBox.counter.BG:SetAllPoints()	
	self.editBox.counter.number = self.editBox.counter:CreateFontString(nil, "OVERLAY")
	self.editBox.counter.number:SetFontObject("SystemFont_Small")
	self.editBox.counter.number:SetPoint("CENTER")
	self.editBox.counter:Hide()

	self.editBoxScroller:SetScrollChild(self.editBox)
	
	---------------
	-- HEADER
	---------------	
	
	self.header.closeBtn:SetScript("OnClick", function(...) self:Hide() end)
	
	self.header.optionsBtn = CreateFrame("Button", nil, self.header)
	self.header.optionsBtn:SetSize(16, 16)
	self.header.optionsBtn:SetPoint("LEFT", 16, 0)
	self.header.optionsBtn:SetNormalTexture([[Interface\Worldmap\Gear_64Grey]])
	self.header.optionsBtn:SetScript("OnClick", function() Core.ShowOptions() end)
	self.header.optionsBtn.tipTitle = "Quick Options"
	Core.utils.RegisterForTooltip(self.header.optionsBtn)
	
	self.header.statusIcon = CreateFrame("Button", nil, self.header)
	self.header.statusIcon:SetSize(16, 16)
	self.header.statusIcon:SetPoint("LEFT")
	self.header.statusIcon:RegisterEvent("PLAYER_FLAGS_CHANGED")
	self.header.statusIcon:RegisterEvent("PLAYER_ENTERING_WORLD")
	self.header.statusIcon:SetScript("OnEvent", statusIcon_OnEvent)
	Core.utils.RegisterForTooltip(self.header.statusIcon)
	
	self.header.title = self.header:CreateFontString(nil)
	self.header.title:SetFont([[Fonts\MORPHEUS.ttf]], 20, "OUTLINE")
	self.header.title:SetPoint("LEFT", 115, 0)
	self.header.title:SetPoint("RIGHT", self.header.closeBtn, "LEFT")
	self.header.title:SetJustifyH("LEFT")
	self.header.title:SetMaxLines(1)
	
	self.header.selectiveBtn = CreateFrame("CheckButton", nil, self.header)
	self.header.selectiveBtn:SetPoint("RIGHT", self.header.title, "LEFT")
	self.header.selectiveBtn:SetSize(16, 16)
	self.header.selectiveBtn:SetNormalTexture([[Interface\COMMON\VOICECHAT-SPEAKER]])
	self.header.selectiveBtn:SetCheckedTexture([[Interface\COMMON\VOICECHAT-MUTED]])
	self.header.selectiveBtn:SetScript("OnClick", function(this) Core.SM.selected.selectiveModeEnabled = this:GetChecked(); end)
	self.header.selectiveBtn.tipTitle = "Show All Messages"
	self.header.selectiveBtn.tipBody = "If disabled, only messages from players whom you have\nsessions opened for will be shown; others will be minimized."
	Core.utils.RegisterForTooltip(self.header.selectiveBtn)
	self.header.selectiveBtn:Hide()
	
	self.header.addBtn = CreateFrame("Button", nil, self.header)
	self.header.addBtn:SetSize(16, 16)
	self.header.addBtn:RegisterForClicks("AnyUp")
	self.header.addBtn:SetNormalTexture([[Interface\PaperDollInfoFrame\Character-Plus]])
	self.header.addBtn:SetScript("OnClick", function() Core.OpenSessionDialog() end)
	self.header.addBtn:SetPoint("LEFT", self.header.optionsBtn, "RIGHT", 2, 0)
	self.header.addBtn:Hide()
	
	self.header.test = CreateFrame("Button", nil, self.header)
	self.header.test:SetNormalAtlas("common-button-square-gray-up")
	self.header.test:SetSize(32, 32)
	self.header.test:SetPoint("Center")

	---------------
	-- BODY
	---------------
	
	self.body = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.body:SetPoint("BOTTOMLEFT", self.lowerFrame, "TOPLEFT")
	self.body:SetPoint("BOTTOMRIGHT", self.lowerFrame, "TOPRIGHT")
	self.body:SetPoint("TOP")
	self.body:SetBackdrop({
		--edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
	})
	
	
	---------------
	-- LEFT STUFF
	---------------
	
	self.leftFrame = CreateFrame("Frame", nil, self, "BackdropTemplate")
	self.leftFrame:SetSize(110, self:GetHeight())
	self.leftFrame:SetPoint("TOPLEFT", self, "TOPLEFT")
	self.leftFrame:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 0, 1)
	self.leftFrame:SetBackdrop({
		--edgeFile=[[Interface\FriendsFrame\UI-Toast-Border]],
		edgeSize=8,
	})
	
	self.tabWindow = CreateFrame("ScrollFrame", nil, self.leftFrame)
	self.tabWindow:SetSize(110, self.leftFrame:GetHeight())
	self.tabWindow:SetPoint("TOP", 0, -6)
	self.tabWindow:SetPoint("BOTTOMRIGHT", 0, 5)
	self.tabWindow:EnableMouseWheel(true)
	self.tabWindow:SetScript("OnVerticalScroll", UI.RefreshTabContainer)
	self.tabWindow:SetScript("OnMouseWheel", function(this, delta)
		local vs = math.floor(this:GetVerticalScroll())
		local range = this:GetVerticalScrollRange()
		local down = (delta == -1)

		if (down and (range <= 0 or vs > range)) or (not down and vs == 0) then 
			return
		end

		this:SetVerticalScroll(down and vs + Core.TAB_TOTALHEIGHT or vs - Core.TAB_TOTALHEIGHT)
		UI.RefreshTabContainer()
	end)

	self.tabContainer = CreateFrame("Frame", nil, self.tabWindow)
	self.tabContainer:SetSize(110, 1)
	self.tabContainer:SetAllPoints()
	self.tabWindow:SetScrollChild(self.tabContainer)
	
	local function createArrow (point)
		local isTop = point == "TOP"
		local f = CreateFrame("Frame", nil, self.leftFrame)
		f:SetSize(102, 8)
		f:SetFrameLevel(6)
		f:SetPoint(point, 0, isTop and -3 or 3)
		f.BG = f:CreateTexture(nil)
		f.BG:SetAllPoints()
		f.BG:SetColorTexture(0, 0, 0, 1)
		f.Tex = f:CreateTexture(nil)
		f.Tex:SetPoint("CENTER", 0, isTop and 2 or -3)
		f.Tex:SetSize(12, 12)
		f.Tex:SetTexture(format([[Interface\BUTTONS\Arrow-%s-Up]], isTop and "Up" or "Down"))
		
		f:Hide()
		return f
	end
	
	--self.leftFrame.upArrow = createArrow("TOP")
	--self.leftFrame.downArrow = createArrow("BOTTOM")

	---------------
	-- OTHER STUFF
	---------------
	
	self.resizeBtn = CreateFrame("Button", nil, self)
	self.resizeBtn:SetSize(16, 16)
	self.resizeBtn:SetPoint("BOTTOMRIGHT", 10, -10)
	self.resizeBtn:SetNormalTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-UP]])
	self.resizeBtn:SetHighlightTexture([[Interface\CHATFRAME\UI-ChatIM-SizeGrabber-Highlight]])
	self.resizeBtn:RegisterForDrag("LeftButton")
	self.resizeBtn:Hide()
	self.resizeBtn:SetScript("OnDragStart", function() self:StartSizing("BOTTOMRIGHT") end)
	self.resizeBtn:SetScript("OnDragStop", function()
		self:StopMovingOrSizing()
		self.RefreshTabContainer()
		self.editBox:SetWidth(self.lowerFrame:GetWidth() -27)
		self.editBoxScroller:SetVerticalScroll(0)
		Core.db.UILayout.width, Core.db.UILayout.height = self:GetSize()
	end)
	
	self.floatingMenu = CreateFrame("Frame", ADDON_NAME.."_FloatingFrame", self, "UIDropDownMenuTemplate")
	
	---[[
	local floatingFrame = CreateFrame("Frame", nil, self, "BackdropTemplate")
	floatingFrame:SetPoint("TOPLEFT", 0, 15)
	floatingFrame:SetPoint("BOTTOMRIGHT")
	floatingFrame:SetBackdrop({
		bgFile=[[Interface\DialogFrame\UI-DialogBox-Background-Dark]],
		edgeFile=[[Interface\DialogFrame\UI-DialogBox-Gold-Border]],
		edgeSize=24,
	})
	floatingFrame:SetFrameStrata("DIALOG")
	floatingFrame.close = CreateFrame("Button", nil, floatingFrame, "UIPanelButtonTemplate")
	floatingFrame.close:SetText("Okay")
	floatingFrame.close:SetPoint("BOTTOM", 0, 10)
	floatingFrame.close:SetScript("OnClick", function() floatingFrame:Hide() end)
	floatingFrame:Hide()
	--]]
	
end